package com.fxkc.care.filter;

import com.alibaba.fastjson.JSONObject;
import com.fxkc.care.common.AjaxResult;
import com.fxkc.care.config.CustomRequestWrapper;
import com.fxkc.care.enums.CommonEnum;
import com.fxkc.care.utils.*;
import com.fxkc.care.vo.UserInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.Base64Utils;

import javax.annotation.Resource;
import javax.servlet.*;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

@WebFilter(filterName = "gatewayFilter", description = "网关过滤器：jwt的校验、白名单资源的处理", urlPatterns = "/*")
@Order(1)
@Slf4j
public class GatewayFilter implements Filter {
    private static final PathMatcherUtil pathMatcherUtil = new AntPathMatcher();
    private String filterName;
    @Resource
    private RedisTemplate redisTemplate;
    @Value("${whiteUrlList}")
    private String whiteUrlList;

    @Override
    public void init(FilterConfig filterConfig) {
        filterName = filterConfig.getFilterName();
        log.info("{} Filter Ready Init...", filterName);
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {

            HttpServletRequest req = (HttpServletRequest) request;
            String servletPath = req.getServletPath();
            String method = req.getMethod();
            CustomRequestWrapper wrapperRequest = new CustomRequestWrapper(req);

            //1.打印记录API请求
            String parameterMapString = "";
            if ("get".equalsIgnoreCase(method)) {
                Map<String, String[]> parameterMap = request.getParameterMap();
                parameterMapString = JSONObject.toJSON(parameterMap).toString();
            } else {
                parameterMapString = wrapperRequest.getBody();
            }
            String ipAddr = IpUtil.getIpAddr(req);
            log.debug("用户[{}]请求API=[{}][{}]，请求参数=[{}]", ipAddr, method, servletPath, parameterMapString);

            //2.URL白名单的过滤放行
            if (whiteUrlList != null && whiteUrlList.trim().length() > 0) {
                for (String whiteUrl : whiteUrlList.split(",")) {
                    String uriPattern = whiteUrl.trim();
                    if (pathMatcherUtil.match(uriPattern, servletPath)) {
                        chain.doFilter(wrapperRequest, response);
                        return;
                    }
                }
            }
            //3.从请求头获取首部字段userToken并校验
            String userToken = Optional.ofNullable(req.getHeader(CommonEnum.USER_TOKEN.getCode())).orElse(request.getParameter(CommonEnum.USER_TOKEN.getCode()));
            try {
                AjaxResult checkResult = JwtUtil.checkJwt(userToken);
                if (checkResult.getCode() != AjaxResult.Type.SUCCESS.getCode()) {
                    log.error("jwt校验不通过:{}", checkResult);
                    ((HttpServletResponse) response).sendError(checkResult.getCode(), checkResult.getErrorMsg().toString());
                } else {
                    //从token中获取loginName
                    String requestJwtPayload = userToken.split("\\.")[1];
                    JSONObject payloadJSON = (JSONObject) JSONObject.parse(Base64Utils.decode(requestJwtPayload.getBytes()));
                    String loginName = (String) payloadJSON.get("loginName");
                    //重置token有效时间
                    redisTemplate.opsForValue().getOperations().expire("jwtsalt:" + loginName, 24, TimeUnit.HOURS);
                    //从redis中获取UserInfo放入Request.attributes中，供上下文用
                    Map<String, Object> userInfoMap = (Map<String, Object>) redisTemplate.opsForHash().get(
                            CommonEnum.USER_INFO.getCode(), loginName);
                    UserInfoVO userInfoVO = (UserInfoVO) MapBeanConvertUtil.convertMap2Bean(UserInfoVO.class, userInfoMap);
                    request.setAttribute(CommonEnum.USER_INFO.getCode(), userInfoVO);
                    chain.doFilter(wrapperRequest, response);
                }
            } catch (Exception e) {
                log.error("校验jwt异常！", e);
                ((HttpServletResponse) response).sendError(AjaxResult.Type.INVALID_TOKEN.getCode(), "token不合法Exception！");//不合法的token
            }
    }

    @Override
    public void destroy() {
        log.info("{}过滤器准备销毁...", filterName);
    }
}
