package com.fxkc.care.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fxkc.care.common.AjaxResult;
import com.fxkc.care.domain.UserInfo;
import com.fxkc.care.domain.UserRole;
import com.fxkc.care.dto.UserInfoAddDTO;
import com.fxkc.care.dto.UserInfoEditDTO;
import com.fxkc.care.enums.CommonEnum;
import com.fxkc.care.enums.DeleteFlagEnum;
import com.fxkc.care.mapper.UserInfoMapper;
import com.fxkc.care.mapper.UserRoleMapper;
import com.fxkc.care.service.IUserInfoService;
import com.fxkc.care.utils.*;
import com.fxkc.care.vo.UserInfoVO;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author JYH
 * @since 2024-12-14
 */
@Slf4j
@Service
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements IUserInfoService {

    @Autowired
    private UserRoleMapper userRoleMapper;

    //密码加密公钥
    @Value("${rsa.publicKey}")
    private String rsaPublicKey;

    //密码加密私钥
    @Value("${rsa.privateKey}")
    private String rsaPrivateKey;

    //账号默认密码
    @Value("${user.defaultPassword}")
    private String defaultPassword;

    @Resource
    private RedisTemplate redisTemplate;

    /**
     * 用户登录
     *
     * @param userInfoVO
     * @param response
     * @return
     */
    @Override
    public AjaxResult doLogin(UserInfoVO userInfoVO, HttpServletResponse response) {
        //1.非空校验
        Assert.notNull(userInfoVO.getLoginName(), "用户名不得为空");
        Assert.notNull(userInfoVO.getPassword(), "密码不得为空");
        UserInfo userInfoDB = this.baseMapper.selectOne(Wrappers.<UserInfo>lambdaQuery()
                .eq(UserInfo::getLoginName, userInfoVO.getLoginName())
                .eq(UserInfo::getDeleteFlag, DeleteFlagEnum.NOT_DELETE.getValue())
        );
        //2.判断用户是否存在
        if (ObjectUtils.isEmpty(userInfoDB)) {
            return AjaxResult.error401("用户名错误！");
        }
        //3.数据库密码解密、前端密码解密
        String password = RSAUtils.decrypt(userInfoDB.getPassword(), this.rsaPrivateKey);
        String inputPassword = RSAUtils.decrypt(userInfoVO.getPassword(), this.rsaPrivateKey);
        if (!StringUtils.equals(password, inputPassword)) {
            return AjaxResult.error401("输入密码错误！");
        }
        //4.生成JWT
        String salt = userInfoDB.getSalt();
        Map<String, Object> jwtPayloadUserInfo = new HashMap<>();
        jwtPayloadUserInfo.put("loginName", userInfoDB.getLoginName());
        jwtPayloadUserInfo.put("loginTime", MultiThreadDateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
        long nowMilliSeconds = System.currentTimeMillis();
        jwtPayloadUserInfo.put("timestamp", nowMilliSeconds);//该字段用来验证jwt的有效性，和redis的jwtsalt[1]比较
        String jwt = JwtUtil.createJwt(jwtPayloadUserInfo, salt);
        //5.保存(覆盖)用户信息、salt_timestamp到redis中
        redisTemplate.opsForValue().set("jwtsalt:" + userInfoVO.getLoginName(), salt + "." + nowMilliSeconds);
        redisTemplate.opsForValue().getOperations().expire("jwtsalt:" + userInfoVO.getLoginName(), 24, TimeUnit.HOURS);
        BeanUtils.copyProperties(userInfoDB, userInfoVO);
        userInfoVO.setPassword("");
        redisTemplate.opsForHash().put(CommonEnum.USER_INFO.getCode(), userInfoDB.getLoginName(), userInfoVO);
        //6.response.cookie加入JWT，重定向到首页
        Cookie cookie = new Cookie(CommonEnum.USER_TOKEN.getCode(), jwt);
        cookie.setPath("/");//这里需要设置，否则跳转页面时浏览器不能写入cookie到域名下
        response.addCookie(cookie);
        //7.记录该用户的登录时间到redis中
        redisTemplate.opsForValue().set(userInfoDB.getUsername() + "_loginTime", String.valueOf(nowMilliSeconds));
        redisTemplate.opsForValue().getOperations().expire(userInfoDB.getUsername() + "_loginTime", 24, TimeUnit.HOURS);
        log.info("用户{}登录成功！", userInfoVO.getLoginName());
        return AjaxResult.success("登录成功！");
    }

    /**
     * 退出登录
     *
     * @return
     */
    @Override
    public AjaxResult logout() {
        // 从Request中获取UserInfo，根据loginName从redis中删除该用户的jwtsalt
        UserInfoVO userInfo = RequestUtil.getUserInfo();
        redisTemplate.opsForValue().getOperations().delete("jwtsalt:" + userInfo.getLoginName());
        log.info("用户{}登出成功！", userInfo.getLoginName());
        return AjaxResult.success("登出成功！");
    }

    /**
     * 新增用户
     *
     * @param dto
     * @return
     */
    @SneakyThrows
    @Override
    public AjaxResult add(UserInfoAddDTO dto) {
        List<UserInfo> userList = new ArrayList<>();
        // 校验手机号
        if (StringUtils.isNotBlank(dto.getPhone())) {
            if (!ValidatorUtil.isMobile(dto.getPhone())) {
                return AjaxResult.error("手机号码格式错误！");
            }
            userList = this.baseMapper.selectList(new LambdaQueryWrapper<UserInfo>()
                    .eq(UserInfo::getDeleteFlag, DeleteFlagEnum.NOT_DELETE.getValue())
                    .eq(UserInfo::getPhone, dto.getPhone()));
            if (CollectionUtils.isNotEmpty(userList)) {
                return AjaxResult.error("手机号码已存在，请更换其他手机号码！");
            }
        }
        // 校验身份证号
        if (StringUtils.isNotBlank(dto.getIdCardNo())) {
            if (!ValidatorUtil.isIdCard(dto.getIdCardNo())) {
                return AjaxResult.error("身份证格式错误！");
            }
            userList = this.baseMapper.selectList(new LambdaQueryWrapper<UserInfo>()
                    .eq(UserInfo::getDeleteFlag, DeleteFlagEnum.NOT_DELETE.getValue())
                    .eq(UserInfo::getIdCardNo, dto.getIdCardNo()));
            if (CollectionUtils.isNotEmpty(userList)) {
                return AjaxResult.error("身份证号码已存在，请更换其他身份证号码！");
            }
        }
        // 判断登录用户名是否存在
        userList = this.baseMapper.selectList(new LambdaQueryWrapper<UserInfo>()
                .eq(UserInfo::getDeleteFlag, DeleteFlagEnum.NOT_DELETE.getValue())
                .eq(UserInfo::getLoginName, dto.getLoginName()));
        if (CollectionUtils.isNotEmpty(userList)) {
            return AjaxResult.error("登录名已存在，请更换其他登录名！");
        }

        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(dto, userInfo);
        // 获取初始密码
        String password = RSAUtils.encrypt(this.defaultPassword, this.rsaPublicKey);
        // 获取盐
        String salt = UuidUtil.getUUID();
        userInfo.setPassword(password);
        userInfo.setSalt(salt);
        this.save(userInfo);
        return AjaxResult.success("新增成功！");
    }

    /**
     * 编辑
     *
     * @param dto
     * @return
     */
    @Override
    public AjaxResult edit(UserInfoEditDTO dto) {
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(dto, userInfo);
        this.updateById(userInfo);
        return AjaxResult.success("编辑成功！");
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public AjaxResult delete(String id) {
        // 删除用户信息
        this.delete(id);
        // 删除用户角色中间表信息
        userRoleMapper.delete(Wrappers.<UserRole>lambdaQuery().eq(UserRole::getUserId, id));
        return AjaxResult.success("删除成功！");
    }
}
