package com.fxkc.care.utils;

import java.util.regex.Pattern;

/**
 * 校验器工具类
 *
 * @author shexuegang
 * @since 2023-11-30
 */
public class ValidatorUtil {
    /**
     * 正则表达式：验证用户名
     */
    private static final String REGEX_USERNAME = "^[a-zA-Z]\\w{5,17}$";

    /**
     * 正则表达式：验证密码
     */
    private static final String REGEX_PASSWORD = "^(?:(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])|(?=.*[A-Z])(?=.*[a-z])(?=.*[\\W_])|(?=.*[A-Z])(?=.*[0-9])(?=.*[\\W_])|(?=.*[a-z])(?=.*[0-9])(?=.*[\\W_])).+$";

    /**
     * 正则表达式：验证手机号
     */
    private static final String REGEX_MOBILE = "^((13[0-9])|(14[5,7,9])|(15[0-3,5-9])|(166)|(17[3,5,6,7,8])|(18[0-9])|(19[8,9]))\\d{8}$";

    /**
     * 正则表达式：验证邮箱
     */
    private static final String REGEX_EMAIL = "^([a-z0-9A-Z]+[-|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$";

    /**
     * 正则表达式：验证汉字
     */
    private static final String REGEX_CHINESE = "^[\u4e00-\u9fa5],{0,}$";

    /**
     * 正则表达式：验证身份证
     */
    private static final String REGEX_ID_CARD = "(^\\d{15}$)|(^\\d{17}([0-9]|X)$)";

    /**
     * 正则表达式：验证网络地址
     */
    private static final String REGEX_URL = "http(s)?://([\\w-]+\\.)+[\\w-]+(/[\\w- ./?%&=]*)?";

    /**
     * 正则表达式：验证接口地址
     */
    private static final String REGEX_INTERFACE_URL = "http(s)?://(([a-zA-Z0-9\\-\\.]+\\.[a-zA-Z0-9]{2,})|((25[0-5]|(2[0-4]|1\\d|[1-9]|)\\d)\\.?\\b){4})(:[0-9]{1,5})?([\\S]*)?";

    /**
     * 正则表达式：验证IP地址
     */
    private static final String REGEX_IP_ADDR = "(25[0-5]|2[0-4]\\d|[0-1]\\d{2}|[1-9]?\\d)";

    /**
     * 正则表达式：验证是否包含大写字母
     */
    private static final String REGEX_CONTAINS_UPPERCASE_LETTERS = "^(?=.*[A-Z]).+$";

    /**
     * 正则表达式：验证是否包含小写字母
     */
    private static final String REGEX_CONTAINS_LOWERCASE_LETTERS = "^(?=.*[a-z]).+$";

    /**
     * 正则表达式：验证是否包含数字
     */
    private static final String REGEX_CONTAINS_NUMBERS = "^(?=.*[0-9]).+$";

    /**
     * 正则表达式：验证是否包含特殊字符
     */
    private static final String REGEX_CONTAINS_SPECIAL_CHARACTERS = "^.*[!@#$%^&*()_+=\\-].*$";

    /**
     * 校验用户名
     *
     * @param username 用户名
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isUsername(String username) {
        return Pattern.matches(REGEX_USERNAME, username);
    }

    /**
     * 校验密码
     *
     * @param password 密码
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isPassword(String password) {
        return Pattern.matches(REGEX_PASSWORD, password);
    }

    /**
     * 校验手机号
     *
     * @param mobile 手机号
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isMobile(String mobile) {
        return Pattern.matches(REGEX_MOBILE, mobile);
    }

    /**
     * 校验邮箱
     *
     * @param email 邮箱
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isEmail(String email) {
        return Pattern.matches(REGEX_EMAIL, email);
    }

    /**
     * 校验汉字
     *
     * @param chinese 汉字
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isChinese(String chinese) {
        return Pattern.matches(REGEX_CHINESE, chinese);
    }

    /**
     * 校验身份证号
     *
     * @param idCard 身份证号
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isIdCard(String idCard) {
        return Pattern.matches(REGEX_ID_CARD, idCard);
    }

    /**
     * 校验网络地址
     *
     * @param url 网络地址
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isUrl(String url) {
        return Pattern.matches(REGEX_URL, url);
    }

    /**
     * 校验接口地址
     *
     * @param interfaceUrl 接口地址
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isInterfaceUrl(String interfaceUrl) {
        return Pattern.matches(REGEX_INTERFACE_URL, interfaceUrl);
    }

    /**
     * 校验IP地址
     *
     * @param ipAddr IP地址
     * @return 校验通过返回true，否则返回false
     */
    public static boolean isIPAddr(String ipAddr) {
        return Pattern.matches(REGEX_IP_ADDR, ipAddr);
    }

    /**
     * 校验是否包含大写字母
     *
     * @param text 待校验文本
     * @return 校验通过返回true，否则返回false
     */
    public static boolean containsUpperCaseLetters(String text) {
        return Pattern.matches(REGEX_CONTAINS_UPPERCASE_LETTERS, text);
    }

    /**
     * 校验是否包含小写字母
     *
     * @param text 待校验文本
     * @return 校验通过返回true，否则返回false
     */
    public static boolean containsLowerCaseLetters(String text) {
        return Pattern.matches(REGEX_CONTAINS_LOWERCASE_LETTERS, text);
    }

    /**
     * 校验是否包含数字
     *
     * @param text 待校验文本
     * @return 校验通过返回true，否则返回false
     */
    public static boolean containsNumbers(String text) {
        return Pattern.matches(REGEX_CONTAINS_NUMBERS, text);
    }

    /**
     * 校验是否包含特殊字符
     *
     * @param text 待校验文本
     * @return 校验通过返回true，否则返回false
     */
    public static boolean containsSpecialCharacters(String text) {
        return Pattern.matches(REGEX_CONTAINS_SPECIAL_CHARACTERS, text);
    }
}
